==> 0.1.0.tar.gz: lib/src/matchers/collection_matchers.dart <==
// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

class _EmptyCollection implements Matcher<Collection> {
  bool matches(Collection collection) => collection.isEmpty();
  String get description => "be empty";
}

class _CollectionContains<T> implements Matcher<Collection<T>> {
  T obj;

  _CollectionContains(T this.obj);

  bool matches(Collection<T> collection) => collection.some((e) => e == obj);
  String get description => "contain '$obj'";
}

class _EveryElement<T> implements Matcher<Collection<T>> {
  Matcher<T> _matcher;

  _EveryElement(Matcher<T> this._matcher);

  bool matches(Collection<T> collection) => collection.every((e) => _matcher.matches(e));
  String get description => "every element ${_matcher.description}";
}

class _SomeElement<T> implements Matcher<Collection<T>> {
  Matcher<T> _matcher;

  _SomeElement(Matcher<T> this._matcher);

  bool matches(Collection<T> collection) => collection.some((e) => _matcher.matches(e));
  String get description => "some element ${_matcher.description}";
}

class _CollectionLength implements Matcher<Collection> {
  Matcher<int> _matcher;

  _CollectionLength(Matcher<int> this._matcher);

  bool matches(Collection collection) => _matcher.matches(collection.length);
  String get description => "collection length ${_matcher.description}";
}

class _OrderedEquals<T> implements Matcher<Iterable<T>>, HasRichMessage<Iterable<T>> {
  List<T> _expected;

  _OrderedEquals(List<T> this._expected);

  bool matches(Iterable<T> actualIterable) {
    List<T> actual = new List<T>.from(actualIterable);

    if (_expected.length != actual.length) {
      return false;
    }

    for (int i = 0; i < _expected.length; i++) {
      if (_expected[i] != actual[i]) {
        return false;
      }
    }

    return true;
  }

  String get description => "be equal to $_expected (ordered)";

  String buildMessage(Iterable<T> actual)
      => _diffLists(new List<T>.from(_expected), new List<T>.from(actual));
}

class _UnorderedEquals<T> implements Matcher<Iterable<T>> {
  Set<T> _expected;

  _UnorderedEquals(Set<T> this._expected);

  bool matches(Iterable<T> actualIterable) {
    Set<T> actual = new Set<T>.from(actualIterable);

    Set<T> missing = new Set<T>.from(_expected);
    missing.removeAll(actual);
    Set<T> extra = new Set<T>.from(actual);
    extra.removeAll(_expected);

    return missing.isEmpty() && extra.isEmpty();
  }

  String get description => "be equal to $_expected (unordered)";
}

/// Every collection that is empty (i.e., `it.isEmtpy()`) matches.
Matcher<Collection> emptyCollection() => new _EmptyCollection();

/// Every collection that contains given [obj] (i.e., `it.contains(obj)`)
/// matches.
Matcher<Collection/*<T>*/> collectionContains(/*T*/ obj) => new _CollectionContains/*<T>*/(obj);

/// Every collection in which all elements match given [matcher] matches.
Matcher<Collection/*<T>*/> everyElement(Matcher/*<T>*/ matcher) => new _EveryElement/*<T>*/(matcher);

/// Every collection in which at least one element matches given [matcher]
/// matches.
Matcher<Collection/*<T>*/> someElement(Matcher/*<T>*/ matcher) => new _SomeElement/*<T>*/(matcher);

/// Every collection whose length matches given [matcher] matches.
Matcher<Collection> collectionLength(Matcher<int> matcher) => new _CollectionLength(matcher);

/// Every collection which has the same length as [expected] collection and
/// contains all elements of the [expected] collection in the same order
/// matches. Note that both the iterables (expected and actual) can be
/// iterated many times!
Matcher<Iterable/*<T>*/> orderedEquals(Iterable/*<T>*/ expected) => new _OrderedEquals(new List/*<T>*/.from(expected));

/// Every collection which has the same length as [expected] collection and
/// contains all elements of the [expected] collection, regardless of the order,
/// matches. Note that both the iterables (expected and actual) can be
/// iterated many times!
Matcher<Iterable/*<T>*/> unorderedEquals(Iterable/*<T>*/ expected) => new _UnorderedEquals(new Set/*<T>*/.from(expected));

==> 0.1.1.tar.gz: lib/src/matchers/collection_matchers.dart <==
// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of matchers;

class _EmptyCollection implements Matcher<Collection> {
  bool matches(Collection collection) => collection.isEmpty;
  String get description => "be empty";
}

class _CollectionContains<T> implements Matcher<Collection<T>> {
  T obj;

  _CollectionContains(this.obj);

  bool matches(Collection<T> collection) => collection.some((e) => e == obj);
  String get description => "contain '$obj'";
}

class _EveryElement<T> implements Matcher<Collection<T>> {
  Matcher<T> _matcher;

  _EveryElement(this._matcher);

  bool matches(Collection<T> collection) => collection.every((e) => _matcher.matches(e));
  String get description => "every element ${_matcher.description}";
}

class _SomeElement<T> implements Matcher<Collection<T>> {
  Matcher<T> _matcher;

  _SomeElement(this._matcher);

  bool matches(Collection<T> collection) => collection.some((e) => _matcher.matches(e));
  String get description => "some element ${_matcher.description}";
}

class _CollectionLength implements Matcher<Collection> {
  Matcher<int> _matcher;

  _CollectionLength(this._matcher);

  bool matches(Collection collection) => _matcher.matches(collection.length);
  String get description => "collection length ${_matcher.description}";
}

class _OrderedEquals<T> implements Matcher<Iterable<T>>, HasRichMessage<Iterable<T>> {
  List<T> _expected;

  _OrderedEquals(this._expected);

  bool matches(Iterable<T> actualIterable) {
    List<T> actual = new List<T>.from(actualIterable);

    if (_expected.length != actual.length) {
      return false;
    }

    for (int i = 0; i < _expected.length; i++) {
      if (_expected[i] != actual[i]) {
        return false;
      }
    }

    return true;
  }

  String get description => "be equal to $_expected (ordered)";

  String buildMessage(Iterable<T> actual)
      => _diffLists(new List<T>.from(_expected), new List<T>.from(actual));
}

class _UnorderedEquals<T> implements Matcher<Iterable<T>> {
  Set<T> _expected;

  _UnorderedEquals(this._expected);

  bool matches(Iterable<T> actualIterable) {
    Set<T> actual = new Set<T>.from(actualIterable);

    Set<T> missing = new Set<T>.from(_expected);
    missing.removeAll(actual);
    Set<T> extra = new Set<T>.from(actual);
    extra.removeAll(_expected);

    return missing.isEmpty && extra.isEmpty;
  }

  String get description => "be equal to $_expected (unordered)";
}

/// Every collection that is empty (i.e., `it.isEmtpy()`) matches.
Matcher<Collection> emptyCollection() => new _EmptyCollection();

/// Every collection that contains given [obj] (i.e., `it.contains(obj)`)
/// matches.
Matcher<Collection/*<T>*/> collectionContains(/*T*/ obj) => new _CollectionContains/*<T>*/(obj);

/// Every collection in which all elements match given [matcher] matches.
Matcher<Collection/*<T>*/> everyElement(Matcher/*<T>*/ matcher) => new _EveryElement/*<T>*/(matcher);

/// Every collection in which at least one element matches given [matcher]
/// matches.
Matcher<Collection/*<T>*/> someElement(Matcher/*<T>*/ matcher) => new _SomeElement/*<T>*/(matcher);

/// Every collection whose length matches given [matcher] matches.
Matcher<Collection> collectionLength(Matcher<int> matcher) => new _CollectionLength(matcher);

/// Every collection which has the same length as [expected] collection and
/// contains all elements of the [expected] collection in the same order
/// matches. Note that both the iterables (expected and actual) can be
/// iterated many times!
Matcher<Iterable/*<T>*/> orderedEquals(Iterable/*<T>*/ expected) => new _OrderedEquals(new List/*<T>*/.from(expected));

/// Every collection which has the same length as [expected] collection and
/// contains all elements of the [expected] collection, regardless of the order,
/// matches. Note that both the iterables (expected and actual) can be
/// iterated many times!
Matcher<Iterable/*<T>*/> unorderedEquals(Iterable/*<T>*/ expected) => new _UnorderedEquals(new Set/*<T>*/.from(expected));
